@@ -1,4 +1,4 @@
-SELECT * FROM informix.sysprocedures WHERE type = 'P'
+SELECT * FROM informix.sysprocedures --WHERE type = 'P'
 { -- to get the body, can we concat this?
 select data as d from informix.sysprocbody
 where datakey = 'T'
