@@ -1,4 +1,4 @@
 select distinct index_name
-from   dba_indexes
+from   all_indexes
 where  UPPER(owner) = UPPER( ? )
 order by 1
