@@ -1,4 +1,4 @@
 select db_link
-from   dba_db_links
+from   all_db_links
 where  UPPER(owner) = UPPER( ? )
 order by 1
