@@ -1,3 +1,3 @@
 select distinct sequence_owner
-from   dba_sequences
+from   all_sequences
 order by sequence_owner
