@@ -1,4 +1,4 @@
 select sequence_name
-from   dba_sequences
+from   all_sequences
 where  UPPER(sequence_owner) = UPPER( ? )
 order by sequence_name
