@@ -1,4 +1,4 @@
 select master
-from   dba_snapshot_logs
+from   all_snapshot_logs
 where  UPPER(log_owner) = UPPER( ? )
 order by master
