@@ -1,4 +1,4 @@
-select username from dba_users
+select username from all_users
 union
 select 'PUBLIC' from dual
 order by 1
