@@ -1,4 +1,4 @@
 select distinct view_name 
-from dba_views
+from all_views
 where UPPER(owner) = UPPER( ? )
 order by view_name
