@@ -8,9 +8,9 @@ nvl(s.program,'?') "Program",
 nvl(s.machine,'?') "Machine",
 s.process "Foreground Process",
 p.spid "Background Spid"
-from sys.v_$session S,
-sys.v_$process P,
-sys.v_$sqlarea X
+from v$session S,
+v$process P,
+v$sqlarea X
 where s.osuser like lower(nvl( ? ,'%'))
 and s.username like UPPER(nvl( ? ,'%'))
 and s.sid like nvl( ? ,'%')
